"use strict";

(function () {
  var Events, RedisConnection, Scripts, parser;

  parser = require("./parser");

  Events = require("./Events");

  Scripts = require("./Scripts");

  RedisConnection = function () {
    class RedisConnection {
      constructor(options) {
        var Redis;
        Redis = eval("require")("redis"); // Obfuscated or else Webpack/Angular will try to inline the optional redis module
        parser.load(options, this.defaults, this);
        if (this.Events == null) {
          this.Events = new Events(this);
        }
        this.client = Redis.createClient(this.clientOptions);
        this.subClient = Redis.createClient(this.clientOptions);
        this.limiters = {};
        this.shas = {};
        this.ready = new this.Promise((resolve, reject) => {
          var count, done, errorListener;
          count = 0;
          errorListener = e => {
            return this.Events.trigger("error", [e]);
          };
          done = () => {
            if (++count === 2) {
              return resolve({
                client: this.client,
                subscriber: this.subClient
              });
            }
          };
          this.client.on("error", errorListener);
          this.client.once("ready", done);
          this.subClient.on("error", errorListener);
          this.subClient.once("ready", done);
          return this.subClient.on("message", (channel, message) => {
            var ref;
            return (ref = this.limiters[channel]) != null ? ref._store.onMessage(message) : void 0;
          });
        });
      }

      _loadScript(name) {
        return new this.Promise((resolve, reject) => {
          var payload;
          payload = Scripts.payload(name);
          return this.client.multi([["script", "load", payload]]).exec((err, replies) => {
            if (err != null) {
              return reject(err);
            }
            this.shas[name] = replies[0];
            return resolve(replies[0]);
          });
        });
      }

      loadScripts() {
        return this.Promise.all(Scripts.names.map(k => {
          return this._loadScript(k);
        }));
      }

      addLimiter(instance) {
        return new instance.Promise((resolve, reject) => {
          var handler;
          handler = channel => {
            if (channel === instance._channel()) {
              this.subClient.removeListener("subscribe", handler);
              this.limiters[channel] = instance;
              return resolve();
            }
          };
          this.subClient.on("subscribe", handler);
          return this.subClient.subscribe(instance._channel());
        });
      }

      removeLimiter(instance) {
        return delete this.limiters[instance._channel()];
      }

      scriptArgs(name, id, args, cb) {
        var keys;
        keys = Scripts.keys(name, id);
        return [this.shas[name], keys.length].concat(keys, args, cb);
      }

      scriptFn(name) {
        return this.client.evalsha.bind(this.client);
      }

      disconnect(flush) {
        var i, k, len, ref;
        ref = Object.keys(this.limiters);
        for (i = 0, len = ref.length; i < len; i++) {
          k = ref[i];
          this.limiters[k]._store.__disconnect__(flush);
        }
        this.client.end(flush);
        this.subClient.end(flush);
        return this.Promise.resolve();
      }

    };

    RedisConnection.prototype.defaults = {
      clientOptions: {},
      Promise: Promise,
      Events: null
    };

    return RedisConnection;
  }.call(this);

  module.exports = RedisConnection;
}).call(undefined);